<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services;

class CupomDesconto extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    const dataSourceCupom = 'orm00027_cupomDesc',
          dataSourceUnidade = 'orm00027_cupomDesc_filiaisparticipantes',
          dataSourceProdutos = 'orm00027_cupomDesc_produtosparticipantes';
          
    const paramterBagCupom = ['NRORG'],
          paramterBagUnidade = ['NRORG', 'CDOPERADOR', 'CDCUPOMDESCFOS'],
          paramterBagProdutos = ['NRORG', 'CDCUPOMDESCFOS'];
          
    protected $dataSourceName = 'orm00027_cupomDesc';
    protected $parameterBagColumns = ['NRORG'];
	protected $crudService;
    protected $parameterBag;
    protected $CupomDescontoService;
    
    
    public function __construct ($crudService, $parameterBag, $paramBag, Manager $dataSourceManager,  Services\CupomDescontoService $CupomDescontoService) {
        $this->crudService              = $crudService;
        $this->parameterBag             = $parameterBag;
        $this->dataSourceManager        = $dataSourceManager;
		$this->CupomDescontoService     = $CupomDescontoService;
    }
    
    public function find (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceCupom;
        $this->parameterBagColumns = self::paramterBagCupom;
        parent::find($request, $response);     
    }
    
    public function save (DTO\Request $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows()[0];
        $this->dataSourceName      = self::dataSourceCupom;
        $this->parameterBagColumns = self::paramterBagCupom;
        if (isset($row['IDGERARANDON']) && $row['IDGERARANDON'] == 'S') {
            for ($i = 0; $i < $row['RANDON']; $i++) {
                $row['CDCUPOMDESCFOS'] = null;
                while($row['CDCUPOMDESCFOS'] == null){
                    $row['CDCUPOMDESCFOS'] = $this->CupomDescontoService->validRandomCode($this->rand_chars());
                }
                $params = array(':CDCUPOMDESCFOS' => $row['CDCUPOMDESCFOS'],
                                ':DSCUPOMDESCFOS' => $row['DSCUPOMDESCFOS'],
                                ':IDTIPODESC' => $row['IDTIPODESC'],
                                ':VRDESCCUPOM' => $row['VRDESCCUPOM'],
                                ':DTINIVIGENCIA' => $row['DTINIVIGENCIA'],
                                ':DTFIMVIGENCIA' => $row['DTFIMVIGENCIA'],
                                ':IDUSOUNICO' => $row['IDUSOUNICO'],
                                ':NRORG' => $row['NRORG']);
                $this->CupomDescontoService->saveMultiCupons($params);
                $this->saveProdOnRegister($row);
                $this->saveUnidadeOnRegister($row);
            }
        }
        else{   
            parent::save($request, $response);
            $this->saveProdOnRegister($row);
            $this->saveUnidadeOnRegister($row);
        }
    }
    
    public function rand_chars() {
        $c = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
        $l = 10;
        for ($i = 0, $z = strlen($c)-1, $s = $c{rand(0,$z)}, $i = 1; $i != $l; $x = rand(0,$z), $s .= $c{$x}, $s = ($s{$i} == $s{$i-1} ? substr($s,0,-1) : $s), $i=strlen($s));
        return $s;
    }
    
    public function saveProdOnRegister ($row){
        if (isset($row['INTERVALISTA']) && $row['INTERVALISTA'] == 'I'){
            if (empty($row['CDARVPRODF'])){
                $row['CDARVPRODF'] = $row['CDARVPRODI'];
            }
            $params = array(':CDARVPRODF'       => $row['CDARVPRODF'],
                            ':CDARVPRODI'       => $row['CDARVPRODI'],
                            ':CDCUPOMDESCFOS'   => $row['CDCUPOMDESCFOS'],
                            ':NRORG'            => $row['NRORG']);
            $this->CupomDescontoService->saveProdutosParticipanteIntervalo($params);
        }elseif (isset($row['INTERVALISTA']) && $row['INTERVALISTA'] == 'D') {
            $row['PRODNIV'] = str_replace('%', '', str_replace('.', '', $row['PRODNIV'])).'%';
            $params = array(':PRODNIV'          => $row['PRODNIV'],
                            ':CDCUPOMDESCFOS'   => $row['CDCUPOMDESCFOS'],
                            ':NRORG'            => $row['NRORG']);
            $this->CupomDescontoService->saveProdutosParticipanteNivel($params);
        }
        elseif (isset($row['INTERVALISTA']) && $row['INTERVALISTA'] == 'L'){
            foreach ($row['CDPRODUTO'] as $produto) {
                $params = array(':CDPRODLIST'       => $produto,
                                ':CDCUPOMDESCFOS'   => $row['CDCUPOMDESCFOS'],
                                ':NRORG'            => $row['NRORG']);
                $this->CupomDescontoService->saveProdutosParticipanteLista($params);
            }
        }
    }
    
    public function saveUnidadeOnRegister ($row){
        if (isset($row['CDFILIAL']) || !empty($row['CDFILIAL'])) {
            foreach ($row['CDFILIAL'] as $unidade) {
                $params = array(':CDFILIAL' => $unidade,
                                ':CDCUPOMDESCFOS' => $row['CDCUPOMDESCFOS'],
                                ':NRORG' => $row['NRORG']);
                $this->CupomDescontoService->saveUnidadeParticipante($params);
            }
        }
    }
    
    public function findUnidadesParticipantes (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceUnidade;
        $this->parameterBagColumns = self::paramterBagUnidade;
        parent::find($request, $response);     
    }
    
    public function saveUnidadesParticipantes (DTO\Request\DataSet $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows()[0];
        $this->saveUnidadeOnRegister($row);
    }
    
    public function deleteUnidadesParticipantes (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceUnidade;
        $this->parameterBagColumns = self::paramterBagUnidade;
        parent::delete($request, $response);
    }
    
    public function findProdutosParticipantes (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceProdutos;
        $this->parameterBagColumns = self::paramterBagProdutos;
        parent::find($request, $response);
    }
    
    public function saveProdutosParticipantes (DTO\Request\DataSet $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows()[0];
        $this->saveProdOnRegister($row);
    }
    
    public function deleteProdutosParticipantes (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceProdutos;
        $this->parameterBagColumns = self::paramterBagProdutos;
        parent::delete($request, $response);
    }
}