<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Utilities;


class RelVendaPorHora extends CrudWithParameterBag {
    
    protected $parameterBagColumns = array('CDFILIAL', 'HRINI', 'HRFIN', 'DTVENDAINI', 'DTVENDAFIN', 'CDLOJA', 'NRORG', 'TCDLOJA','CDTIPOCONS', 'TCDTIPOCONS', 'MINUTOS' );
    protected $dataSourceName = 'vnd02503_relvendaPorHora';
    
    public function __construct( Manager $dataSourceManager, ParameterBag $parameterBag, Utilities\Util $utilService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->utilService = $utilService;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
       
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if($conditions['CDFILIAL'] !== '' &&  $conditions['CDFILIAL'] !== null) {
            
            if(array_key_exists('DTENTRVENDA', $conditions)) {
                $conditions['DTVENDAINI'] = $conditions['DTENTRVENDA'][0];
                $conditions['DTVENDAFIN'] = $conditions['DTENTRVENDA'][1];
                unset($conditions['DTENTRVENDA']);
            }
            
            if(array_key_exists('HRINI', $conditions) && array_key_exists('HRFIN', $conditions) ) {
                $conditions['HRINI'] = str_replace(":","",$conditions['HRINI']);
                $conditions['HRINI'] = str_replace("%","",$conditions['HRINI']);
                $conditions['HRFIN'] = str_replace(":","",$conditions['HRFIN']);
                $conditions['HRFIN'] = str_replace("%","",$conditions['HRFIN']);
            }
            
            if( !array_key_exists('CDFILIAL', $conditions) ) {
                $conditions['CDFILIAL'] = null;
            }
            
            if( !array_key_exists('CDLOJA', $conditions) ) {
                $conditions['CDLOJA'] = null;
            }
            
            if( !array_key_exists('TCDLOJA', $conditions) ) {
                $conditions['TCDLOJA'] = null;
            }
            
            if( !array_key_exists('DTVENDAINI', $conditions) ) {
                $conditions['DTVENDAINI'] = null;
            }
            
            if( !array_key_exists('DTVENDAFIN', $conditions) ) {
                $conditions['DTVENDAFIN'] = null;
            }
            
            if( !array_key_exists('MINUTOS', $conditions) ) {
                $conditions['MINUTOS'] = null;
            }
            if( !array_key_exists('CDTIPOCONS', $conditions) ) {
                $conditions['CDTIPOCONS'] = null;
            }
             
            if( !array_key_exists('TCDTIPOCONS', $conditions) ) {
                $conditions['TCDTIPOCONS'] = null;
            }
            
            if(array_key_exists('MINUTOS', $conditions)) {
                if(is_null($conditions['MINUTOS'])) {
                    $conditions['MINUTOS'] = '%00:01%';
                } else if($conditions['MINUTOS'] == '%0000%'){
                    $conditions['MINUTOS'] = '%24:00%';
                }else if($conditions['MINUTOS'] == '%00:00%'){
                    $conditions['MINUTOS'] = '%24:00%';
                }
                $conditions['MINUTOS'] = str_replace('%', '', $conditions['MINUTOS']); 
                $time = explode(':', $conditions['MINUTOS']);
                $minutos = $time[0] * 60 + $time[1];
                $conditions['MINUTOS'] = $minutos;
            }
            
            $conditions['CDFILIAL'] = str_replace('%', '', $conditions['CDFILIAL']); 
            
            if(array_key_exists('CDLOJA', $conditions) && $conditions['CDLOJA'] !== null) {
                $conditions['CDLOJA'] = implode('_', $conditions['CDLOJA']);
                $conditions['TCDLOJA'] = null;
            } else {
                $conditions['CDLOJA'] = null;
                $conditions['TCDLOJA'] = 'T';
            }
            if(array_key_exists('CDTIPOCONS', $conditions) && $conditions['CDTIPOCONS'] !== null) {
                $conditions['CDTIPOCONS'] = implode('_', $conditions['CDTIPOCONS']);
                $conditions['TCDTIPOCONS'] = null;
            } else {
                $conditions['CDTIPOCONS'] = null;
                $conditions['TCDTIPOCONS'] = 'T';
            }
            if(!array_key_exists('HRINI', $conditions)){
                $conditions['HRINI'] = 0000;
            } 
            if(!array_key_exists('HRFIN', $conditions)){
                $conditions['HRFIN'] = 2359;
            } 
           
            $dataSourceFilter = array();
            unset($conditions['LABELTIPOCONS']);
            unset($conditions['GROUPTPCONS']);
            foreach($conditions as $key => $value) {
                $dataSourceFilter[] = $condition = array(
                    "columnName" => $key,
                    "operator"=> "=",
                    "value"=> $value
                );
            }
            
            $original = $request->getFilterCriteria()->getConditions();
            
            for($i = 0 ; $i < count($dataSourceFilter) ; $i++) {
                for($j = 0 ; $j < count($original) ; $j++ ) {
                    if( $dataSourceFilter[$i]['columnName'] === $original[$j]['columnName'] &&
                        $dataSourceFilter[$i]['operator'] !== $original[$j]['operator'])  {
                        $dataSourceFilter[$i]['operator'] = $original[$j]['operator'];
                    }
                    if($dataSourceFilter[$i]['columnName'] === 'MINUTOS') {
                        $dataSourceFilter[$i]['operator'] = '=';  
                    }
                    if($dataSourceFilter[$i]['columnName'] === 'HRINI') {
                        $dataSourceFilter[$i]['operator'] = '=';
                    }
                    if($dataSourceFilter[$i]['columnName'] === 'HRFIN') {
                        $dataSourceFilter[$i]['operator'] = '=';
                    }
                }
            }
            $newRequest = new DTO\Request\Filter(
                new FilterCriteria(
                    $request->getFilterCriteria()->getDataSourceName(),
                    $dataSourceFilter,
                    $request->getFilterCriteria()->getPage(),
                    9999999
                ),
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            );
            
            parent::find($newRequest, $response);
            
        }
    }
} 